/************************************************************
 * wd_internal.h
 *
 *   Copyright (C) 2007 Gregory Nutt. All rights reserved.
 *   Author: Gregory Nutt <spudmonkey@racsa.co.cr>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name Gregory Nutt nor the names of its contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ************************************************************/

#ifndef __WD_INTERNAL_H
#define __WD_INTERNAL_H

/************************************************************
 * Included Files
 ************************************************************/

#include <nuttx/config.h>
#include <sys/types.h>
#include <wdog.h>
#include <nuttx/compiler.h>

/************************************************************
 * Compilations Switches
 ************************************************************/

/************************************************************
 * Definitions
 ************************************************************/

/************************************************************
 * Public Type Declarations
 ************************************************************/

/* This is the watchdog structure.  The WDOG_ID is a pointer to
 * a watchdog structure.
 */

struct wdog_s
{
  FAR struct wdog_s *next;       /* Support for singly linked lists. */
  wdentry_t          func;       /* Function to execute when delay expires */
#ifdef CONFIG_NXFLAT
  FAR void          *picbase;    /* PIC base address */
#endif
  int                lag;        /* Timer associated with the delay */
  boolean            active;     /* TRUE if the watchdog is actively timing */
  ubyte              argc;       /* The number of parameters to pass */
  uint32             parm[CONFIG_MAX_WDOGPARMS];
};
typedef struct wdog_s wdog_t;

/************************************************************
 * Public Variables
 ************************************************************/

/* The g_wdfreelist data structure is a singly linked list
 * of watchdogs available to the system for delayed function
 * use.
 */

extern sq_queue_t g_wdfreelist;

/* g_wdpool is a pointer to a list of pre-allocated watchdogs.
 * The number of watchdogs in the pool is a configuration
 * item.
 */

extern FAR wdog_t *g_wdpool;

/* The g_wdactivelist data structure is a singly linked list
 * ordered by watchdog expiration time. When watchdog timers
 * expire,the functions on this linked list are removed and
 * the function is called.
 */

extern sq_queue_t g_wdactivelist;

/************************************************************
 * Public Function Prototypes
 ************************************************************/

#ifdef __cplusplus
#define EXTERN extern "C"
extern "C" {
#else
#define EXTERN extern
#endif

EXTERN void weak_function wd_initialize(void);
EXTERN void weak_function wd_timer(void);

#undef EXTERN
#ifdef __cplusplus
}
#endif

#endif /* __WD_INTERNAL_H */
